#include <windows.h>
#define FN603_Exports
#include "fn603.h"
#include "Cpdio.h"
#include "Wake.h"

typedef char SStr[32];

//---------------------------- Constants: -----------------------------------

//Wake protocol constantes:

#define FRAME     200 //max frame size

//Wake protocol errors:

#define ERR_NO   0    //no error
#define ERR_TX   1    //IO error
#define ERR_BU   2    //device busy error
#define ERR_RE   3    //device not ready
#define ERR_PA   4    //parameters value error
#define ERR_NR   5    //device not responding

//Unknown error:

#define ERR_UK   6    //unknown error

#define MAX_ERR  ERR_UK

//Errors names:

const SStr ErrorMsg[MAX_ERR + 1] =
  { "",
    "invalid packet.",
    "device busy.",
    "device not ready.",
    "invalid parameters.",
    "device not responding.",
    "unknown." };

//Wake protocol commands:

#define CMD_NOP        0 //no operation
#define CMD_ERR        1 //get error code
#define CMD_ECHO       2 //get echo
#define CMD_INFO       3 //get information
#define CMD_SETADDR    4 //set net address
#define CMD_GETADDR    5 //get net address

//Specific FN-603 commands:

#define CMD_GETT       6 //get temperature
#define CMD_GETV       7 //get fan RPM
#define CMD_SETM       8 //set mode
#define CMD_GETM       9 //get mode
#define CMD_SETPWM    10 //set PWM
#define CMD_GETPWM    11 //get PWM

//Unknown command:

#define CMD_UNKNOWN   12 //unknown command

#define MAX_CMD       CMD_UNKNOWN

//Commands names:

const SStr CmdNames[MAX_CMD + 1] =
  { "",
    "IO error: ",
    "Get echo error: ",
    "Get info error: ",
    "Set address error: ",
    "Get address error: ",
    "Get temperature: ",
    "Get fan RPM: ",
    "Set channel mode: ",
    "Get channel mode: ",
    "Set PWM: ",
    "Get PWM: ",
    "System error: " };

const SStr DevName = "FN-603\0"; //device name
const DWORD Baud = 38400;        //baud rate
const ULONG RxTo = 200;          //RX timeout, mS

//------------------------------ Variables: ---------------------------------

HANDLE DevHandle;            //device handle
CRITICAL_SECTION Cs;         //critical section
unsigned char TxData[FRAME]; //TX data buffer
unsigned char RxData[FRAME]; //RX data buffer
unsigned char addr, cmd, n;  //Wake protocol variables
char LastError[64] = "";     //last error string

//--------------------- Functions prototypes: -------------------------------

bool __fastcall CmdExe(unsigned char cmd, unsigned char n);
bool __fastcall ErrorReport(int cmd, int errc);

//---------------------------------------------------------------------------

#pragma argsused

int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void* lpReserved)
{
  switch(reason)
  {
    case DLL_PROCESS_ATTACH: InitializeCriticalSection(&Cs); break;
    case DLL_PROCESS_DETACH: DeleteCriticalSection(&Cs); break;
  }
  return 1;
}

//-------------------------- Execute command: -------------------------------

bool __fastcall CmdExe(unsigned char cmd, unsigned char n)
{
  setmem(RxData, FRAME, 0);                    //clear RX data
  if(DevHandle)                                //if port is opened
   {
     if(TxFrame(0, cmd, n, TxData) &&          //transmit frame
        RxFrame(RxTo, addr, cmd, n, RxData))   //receive frame
          return(ErrorReport(cmd, RxData[0])); //cuccess
     return(ErrorReport(CMD_ERR, ERR_NR));     //device is not respondong
   }
  return(ErrorReport(CMD_NOP, ERR_NO));        //port is closed, return
}

//-------------------------- Report of error: -------------------------------

bool __fastcall ErrorReport(int cmd, int errc)
{
  if(cmd == CMD_ECHO || cmd == CMD_INFO) errc = ERR_NO; //no error code
  if(errc == ERR_NO) cmd = CMD_NOP;  //if no error, empty command string
  if(cmd > MAX_CMD) cmd = MAX_CMD;   //check for command code out of range
  if(errc > MAX_ERR) errc = MAX_ERR; //check for error code out of range
  strcpy(LastError, CmdNames[cmd]);  //copy command name
  strcat(LastError, ErrorMsg[errc]); //copy error message
  return(errc == ERR_NO);
}

//---------------------------------------------------------------------------
//-------------------------- Exported functions -----------------------------
//---------------------------------------------------------------------------

//---------------------------- Open device: ---------------------------------

bool WINAPI FN603_OpenDevice(void)
{
  DWORD numDevs;
  LPCSTR DevInfo;
  if(!ListDevices(numDevs)) return(0);
  for(DWORD i = 0; i < numDevs; i++)
  {
    if(OpenPort(i, DevHandle))
      if(SetupPort(Baud))
        if(FN603_GetInfo(DevInfo))
          if(!strncmp(DevInfo, DevName, 6))
          {
            return(1);
          }
    if(DevHandle)
    {
      ClosePort(DevHandle);
    }
  }
  return(0);
}

//---------------------------- Close device: --------------------------------

bool WINAPI FN603_CloseDevice(void)
{
  return(ClosePort(DevHandle));
}

//------------------------- Get last error name: ----------------------------

void WINAPI FN603_GetLastError(LPCSTR &lpcStr)
{
  lpcStr = LastError;
}

//------------------------------ CMD_INFO: ----------------------------------

bool WINAPI FN603_GetInfo(LPCSTR &lpcStr)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_INFO;
  bool rx = CmdExe(cmd, 0);
  lpcStr = (LPCSTR)RxData;
  LeaveCriticalSection(&Cs);
  return(rx);
}

//------------------------------- C_GetT: -----------------------------------

bool WINAPI FN603_GetT(int &t1, int &t2, int &t3, int &t4, int &t5, int &t6)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_GETT;
  bool rx = CmdExe(cmd, 0);
  t1 = (short)(RxData[1]  | (RxData[2]  << 8));
  t2 = (short)(RxData[3]  | (RxData[4]  << 8));
  t3 = (short)(RxData[5]  | (RxData[6]  << 8));
  t4 = (short)(RxData[7]  | (RxData[8]  << 8));
  t5 = (short)(RxData[9]  | (RxData[10] << 8));
  t6 = (short)(RxData[11] | (RxData[12] << 8));
  LeaveCriticalSection(&Cs);
  return(rx);
}

//------------------------------- C_GetV: -----------------------------------

bool WINAPI FN603_GetV(int &v1, int &v2, int &v3, int &v4, int &v5, int &v6)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_GETV;
  bool rx = CmdExe(cmd, 0);
  v1 = RxData[1]  | (RxData[2]  << 8);
  v2 = RxData[3]  | (RxData[4]  << 8);
  v3 = RxData[5]  | (RxData[6]  << 8);
  v4 = RxData[7]  | (RxData[8]  << 8);
  v5 = RxData[9]  | (RxData[10] << 8);
  v6 = RxData[11] | (RxData[12] << 8);
  LeaveCriticalSection(&Cs);
  return(rx);
}

//------------------------------- C_SetM: -----------------------------------

bool WINAPI FN603_SetM(int chan, int mode, int Tmin, int Pmin,
                       int Tmax, int Pmax, int At, int Av, int Pman)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_SETM;
  TxData[0] =  (unsigned char)  chan & 0xFF;
  TxData[1] =  (unsigned char)  mode & 0xFF;
  TxData[2] =  (unsigned char)  Tmin & 0xFF;
  TxData[3] =  (unsigned char) (Tmin >> 8) & 0xFF;
  TxData[4] =  (unsigned char)  Pmin & 0xFF;
  TxData[5] =  (unsigned char)  Tmax & 0xFF;
  TxData[6] =  (unsigned char) (Tmax >> 8) & 0xFF;
  TxData[7] =  (unsigned char)  Pmax & 0xFF;
  TxData[8] =  (unsigned char)  At & 0xFF;
  TxData[9] =  (unsigned char) (At >> 8) & 0xFF;
  TxData[10] = (unsigned char)  Av & 0xFF;
  TxData[11] = (unsigned char) (Av >> 8) & 0xFF;
  TxData[12] = (unsigned char)  Pman & 0xFF;
  bool rx = CmdExe(cmd, 13);
  LeaveCriticalSection(&Cs);
  return(rx);
}

//------------------------------- C_GetM: -----------------------------------

bool WINAPI FN603_GetM(int chan, int &mode, int &Tmin, int &Pmin,
                       int &Tmax, int &Pmax, int &At, int &Av, int &Pman)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_GETM;
  TxData[0] = (unsigned char) chan & 0xFF;
  bool rx = CmdExe(cmd, 1);
  mode = RxData[1];
  Tmin = RxData[2]  | (RxData[3]  << 8);
  Pmin = RxData[4];
  Tmax = RxData[5]  | (RxData[6]  << 8);
  Pmax = RxData[7];
  At   = RxData[8]  | (RxData[9]  << 8);
  Av   = RxData[10] | (RxData[11] << 8);
  Pman = RxData[12];
  LeaveCriticalSection(&Cs);
  return(rx);
}

//------------------------------ C_SetPWM: ----------------------------------

bool WINAPI FN603_SetPWM(int p1, int p2, int p3, int p4, int p5, int p6)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_SETPWM;
  TxData[0] = (unsigned char) p1 & 0xFF;
  TxData[1] = (unsigned char) p2 & 0xFF;
  TxData[2] = (unsigned char) p3 & 0xFF;
  TxData[3] = (unsigned char) p4 & 0xFF;
  TxData[4] = (unsigned char) p5 & 0xFF;
  TxData[5] = (unsigned char) p6 & 0xFF;
  bool rx = CmdExe(cmd, 6);
  LeaveCriticalSection(&Cs);
  return(rx);
}

//------------------------------ C_GetPWM: ----------------------------------

bool WINAPI FN603_GetPWM(int &p1, int &p2, int &p3, int &p4, int &p5, int &p6)
{
  EnterCriticalSection(&Cs);
  cmd = (unsigned char) CMD_GETPWM;
  bool rx = CmdExe(cmd, 0);
  p1 = RxData[1];
  p2 = RxData[2];
  p3 = RxData[3];
  p4 = RxData[4];
  p5 = RxData[5];
  p6 = RxData[6];
  LeaveCriticalSection(&Cs);
  return(rx);
}

//---------------------------------------------------------------------------

